@@ -2,7 +2,8 @@ import json as jsonlib
 from html import unescape
 
 from django import template
-from django.template.base import FilterExpression, kwarg_re, NodeList
+from django.template import engines, TemplateSyntaxError
+from django.template.base import NodeList, token_kwargs
 from django.template.defaulttags import IfNode
 from django.template.loader import get_template
 from django.template.loader_tags import BlockNode, ExtendsNode
@@ -298,44 +299,40 @@ def handle_rtf_linebreaks(value):
 
 
 def parse_tag(token, parser):
-    """
-    Generic template tag parser.
-
-    Returns a three-tuple: (tag_name, args, kwargs)
-
-    tag_name is a string, the name of the tag.
-
-    args is a list of FilterExpressions, from all the arguments that didn't look like kwargs,
-    in the order they occurred, including any that were mingled amongst kwargs.
-
-    kwargs is a dictionary mapping kwarg names to FilterExpressions, for all the arguments that
-    looked like kwargs, including any that were mingled amongst args.
-
-    (At rendering time, a FilterExpression f can be evaluated by calling f.resolve(context).)
-    """
-    # Split the tag content into words, respecting quoted strings.
     bits = token.split_contents()
-
-    # Pull out the tag name.
     tag_name = bits.pop(0)
-
-    # Parse the rest of the args, and build FilterExpressions from them so that
-    # we can evaluate them later.
-    args = []
-    kwargs = {}
-    for bit in bits:
-        # Is this a kwarg or an arg?
-        match = kwarg_re.match(bit)
-        kwarg_format = match and match.group(1)
-        if kwarg_format:
-            key, value = match.groups()
-            kwargs[key] = FilterExpression(value, parser)
+    args, kwargs = [], {}
+    while bits:
+        bit = bits.pop(0)
+        param = token_kwargs([bit], parser)
+        if param:
+            kwargs.update(param)
         else:
-            args.append(FilterExpression(bit, parser))
+            args.append(parser.compile_filter(bit))
 
     return tag_name, args, kwargs
 
 
+def template_from_string(template_string, using=None):
+    """
+    Convert a string into a template object,
+    using a given template engine or using the default backends
+    from settings.TEMPLATES if no engine was specified.
+    """
+    # This function is based on django.template.loader.get_template,
+    # but uses Engine.from_string instead of Engine.get_template.
+
+    # https://stackoverflow.com/a/46756430/9625282
+    chain = []
+    engine_list = engines.all() if using is None else [engines[using]]
+    for engine in engine_list:
+        try:
+            return engine.from_string(template_string)
+        except TemplateSyntaxError as e:
+            chain.append(str(e))
+    raise TemplateSyntaxError(template_string + ','.join(chain))
+
+
 class ExtendTemplateNode(template.Node):
     """
     Node for rendering extended template
@@ -374,7 +371,9 @@ class ExtendTemplateNode(template.Node):
     def get_node_list(nodelist):
         while len(nodelist) == 1 and isinstance(nodelist[0], ExtendsNode):
             nodelist = nodelist[0].nodelist
-        return nodelist
+        ret = NodeList(nodelist)
+        ret.contains_nontext = nodelist.contains_nontext
+        return ret
 
     def render(self, context):
 
@@ -387,26 +386,35 @@ class ExtendTemplateNode(template.Node):
         except:
             return ''
 
-        if 'block' in kwargs:
-            nodelist = self.get_node_list(extending_template.template.nodelist)
-            block_nodes = self.get_all_blocks(nodelist)
-            extending_template.template.nodelist = NodeList(
-                [node for node in block_nodes if node.name == kwargs.get('block')]
-            )
-            extending_template.template.nodelist.contains_nontext = True
-
-        if self.multiline:
-            blocks = {node.name: node for node in self.nodelist if isinstance(node, BlockNode)}
-            nodelist = self.get_node_list(extending_template.template.nodelist)
-
-            self.rearrange_blocks(nodelist, blocks)
-
         if self.only:
             flattened_context = kwargs
         else:
             flattened_context = context.flatten()
             flattened_context.update(kwargs)
 
+        nodelist_changed = False
+        new_nodelist = self.get_node_list(extending_template.template.nodelist)
+
+        if 'block' in kwargs:
+            block_nodes = self.get_all_blocks(new_nodelist)
+            new_nodelist = NodeList(
+                [node for node in block_nodes if node.name == kwargs.get('block')]
+            )
+            new_nodelist.contains_nontext = True
+            nodelist_changed = True
+
+        if self.multiline:
+            blocks = {node.name: node for node in self.nodelist if isinstance(node, BlockNode)}
+            self.rearrange_blocks(new_nodelist, blocks)
+            nodelist_changed = True
+
+        # This is because there can also be cache loader for template. And if in template loaded with such loader nodes
+        # are changed, they are changed in next iteration also.
+        # So now I load template from empty string and give him changed nodelist. So I don't change original template
+        if nodelist_changed:
+            extending_template = template_from_string('', None)
+            extending_template.template.nodelist = new_nodelist
+
         content = extending_template.render(flattened_context)
         content = mark_safe(content)
         return content
